@@ -1,14 +1,15 @@
 #ifndef SRC_CONFIG_H_
 #define SRC_CONFIG_H_
 
-#include <stdbool.h>
+#include <arpa/inet.h>
+#include <netdb.h>
+#include <netinet/in.h>
 #include <stdint.h>
 
 #include "config/filename_format.h"
 #include "config/log_conf.h"
 #include "config/mode.h"
 #include "config/output_format.h"
-#include "config/rsync_strategy.h"
 #include "config/string_array.h"
 #include "config/types.h"
 
@@ -29,10 +30,10 @@ char const *config_get_slurm(void);
 
 char const *config_get_tal(void);
 char const *config_get_local_repository(void);
-bool config_get_shuffle_tal_uris(void);
 unsigned int config_get_max_cert_depth(void);
 enum mode config_get_mode(void);
 char const *config_get_http_user_agent(void);
+unsigned int config_get_max_redirs(void);
 long config_get_http_connect_timeout(void);
 long config_get_http_transfer_timeout(void);
 long config_get_http_low_speed_limit(void);
@@ -41,11 +42,10 @@ long config_get_http_max_file_size(void);
 char const *config_get_http_ca_path(void);
 bool config_get_rsync_enabled(void);
 unsigned int config_get_rsync_priority(void);
-enum rsync_strategy config_get_rsync_strategy(void);
 unsigned int config_get_rsync_retry_count(void);
 unsigned int config_get_rsync_retry_interval(void);
 char *config_get_rsync_program(void);
-struct string_array const *config_get_rsync_args(bool);
+struct string_array const *config_get_rsync_args(void);
 bool config_get_http_enabled(void);
 unsigned int config_get_http_priority(void);
 unsigned int config_get_http_retry_count(void);
@@ -54,9 +54,7 @@ char const *config_get_output_roa(void);
 char const *config_get_output_bgpsec(void);
 enum output_format config_get_output_format(void);
 unsigned int config_get_asn1_decode_max_stack(void);
-unsigned int config_get_stale_repository_period(void);
 unsigned int config_get_thread_pool_server_max(void);
-unsigned int config_get_thread_pool_validation_max(void);
 
 /* Logging getters */
 bool config_get_op_log_enabled(void);
@@ -75,26 +73,9 @@ uint8_t config_get_val_log_level(void);
 enum log_output config_get_val_log_output(void);
 uint32_t config_get_val_log_facility(void);
 
-/*
- * Public, so that work-offline can set them, or (to be deprecated)
- * sync-strategy when set to 'off'.
- */
+/* Public, so --work-offline can override them. */
 void config_set_rsync_enabled(bool);
 void config_set_http_enabled(bool);
-/* TODO (later) Deprecated */
-void config_set_rrdp_enabled(bool);
-
-/* TODO (later) Remove when sync-strategy is fully deprecated */
-void config_set_sync_strategy(enum rsync_strategy);
-void config_set_rsync_strategy(enum rsync_strategy);
-
-/* TODO (later) Remove once rrdp.* is fully deprecated */
-void config_set_rrdp_priority(unsigned int);
-void config_set_http_priority(unsigned int);
-void config_set_rrdp_retry_count(unsigned int);
-void config_set_http_retry_count(unsigned int);
-void config_set_rrdp_retry_interval(unsigned int);
-void config_set_http_retry_interval(unsigned int);
 
 /* Needed public by the JSON module */
 void *get_rpki_config_field(struct option_field const *);
