@@ -1,8 +1,10 @@
 #include "config/log_conf.h"
 
+#include <arpa/inet.h>
 #include <getopt.h>
-#include <stdlib.h>
-#include <string.h>
+#include <netdb.h>
+#include <netinet/in.h>
+#include <stdint.h>
 #include <syslog.h>
 
 #include "log.h"
