@@ -1,8 +1,6 @@
 #include "config/output_format.h"
 
 #include <getopt.h>
-#include <stdlib.h>
-#include <string.h>
 
 #include "log.h"
 #include "config/str.h"
