@@ -1,49 +1,16 @@
 #include "init.h"
 
-#include <errno.h>
-#include <stdio.h>
-#include <stdbool.h>
-#include <unistd.h>
-
+#include "alloc.h"
+#include "config.h"
 #include "log.h"
 #include "http/http.h"
 
-static bool
-download_arin_tal(void)
-{
-	char c;
-
-	printf("Attention: ARIN requires you to agree to their Relying Party Agreement (RPA) before you can download and use their TAL.\n"
-	    "Please download and read https://www.arin.net/resources/manage/rpki/rpa.pdf\n"
-	    "If you agree to the terms, type 'yes' and hit Enter: ");
-
-	c = getchar();
-	if (c != 'y' && c != 'Y')
-		goto cancel;
-
-	c = getchar();
-	if (c != 'e' && c != 'E')
-		goto cancel;
-
-	c = getchar();
-	if (c != 's' && c != 'S')
-		goto cancel;
-
-	if (feof(stdin) || (c = getchar()) == '\n')
-		return true;
-
-	/* Fall through */
-cancel:
-	printf("Skipping ARIN's TAL.\n\n");
-	return false;
-}
-
 static int
 fetch_url(char const *url)
 {
 	char const *prefix = "https://";
 	char const *dest_dir;
-	char *dest_file;
+	char const *dest_file;
 	char *dest;
 	size_t prefix_len;
 	size_t url_len;
@@ -67,9 +34,7 @@ fetch_url(char const *url)
 
 	extra_slash = (dest_dir[dest_dir_len - 1] == '/') ? 0 : 1;
 
-	dest = malloc(dest_dir_len + extra_slash + strlen(dest_file) + 1);
-	if (dest == NULL)
-		return pr_enomem();
+	dest = pmalloc(dest_dir_len + extra_slash + strlen(dest_file) + 1);
 
 	offset = 0;
 	strcpy(dest + offset, dest_dir);
@@ -99,30 +64,45 @@ download_tals(void)
 {
 	int error;
 
-	/*
-	 * https://afrinic.net/resource-certification/tal
-	 * https://www.apnic.net/community/security/resource-certification/tal-archive/
-	 * https://www.arin.net/resources/manage/rpki/tal/
-	 * https://www.lacnic.net/4984/2/lacnic/rpki-rpki-trust-anchor
-	 * https://www.ripe.net/manage-ips-and-asns/resource-management/rpki/ripe-ncc-rpki-trust-anchor-structure
-	 */
-
+	/* https://afrinic.net/resource-certification/tal */
 	error = fetch_url("https://rpki.afrinic.net/tal/afrinic.tal");
 	if (error)
 		return error;
+
+	/*
+	 * https://www.apnic.net/community/security/resource-certification/tal-archive/
+	 *
+	 * APNIC is a bit weird. Some thoughts:
+	 *
+	 * 1. The 6490 and ripe-validator TALs are obsolete, and Fort has never
+	 *    been compatible with them.
+	 * 2. apnic.tal is identical to apnic-rfc7730.tal, and neither of them
+	 *    contain HTTP URLs.
+	 * 3. apnic-rfc7730-https.tal is not actually compliant with RFC 7730;
+	 *    it's an RFC 8630 TAL. However, I'm wondering if there's a reason
+	 *    why they haven't upgraded it to their default TAL.
+	 *
+	 * I'll stick to the rsync-only one until I've tested it more.
+	 */
 	error = fetch_url("https://tal.apnic.net/apnic.tal");
 	if (error)
 		return error;
-	if (download_arin_tal())
-		error = fetch_url("https://www.arin.net/resources/manage/rpki/arin.tal");
-	error = fetch_url("https://www.lacnic.net/innovaportal/file/4983/1/lacnic.tal");
-	if (error)
-		return error;
-	error = fetch_url("https://tal.rpki.ripe.net/ripe-ncc.tal");
+
+	/* https://www.arin.net/resources/manage/rpki/tal/ */
+	error = fetch_url("https://www.arin.net/resources/manage/rpki/arin.tal");
 	if (error)
 		return error;
 
-	return error;
+	/* https://www.lacnic.net/4984/2/lacnic/rpki-rpki-trust-anchor */
+	error = fetch_url("https://www.lacnic.net/innovaportal/file/4983/1/lacnic.tal");
+	if (error)
+		return error;
+
+	/*
+	 * https://www.ripe.net/manage-ips-and-asns/resource-management/rpki/ripe-ncc-rpki-trust-anchor-structure
+	 * I wish they stated why they don't recommend the 8630 TAL.
+	 */
+	return fetch_url("https://tal.rpki.ripe.net/ripe-ncc.tal");
 }
 
 int
@@ -133,6 +113,5 @@ download_tal0s(void)
 	error = fetch_url("https://tal.apnic.net/apnic-as0.tal");
 	if (error)
 		return error;
-
 	return fetch_url("https://www.lacnic.net/innovaportal/file/4983/1/lacnic-as0.tal");
 }
