@@ -1,9 +1,6 @@
 #include "line_file.h"
 
-#include <err.h>
-#include <errno.h>
-#include <stdlib.h>
-
+#include "alloc.h"
 #include "file.h"
 #include "log.h"
 
@@ -22,9 +19,7 @@ lfile_open(const char *file_name, struct line_file **result)
 	struct line_file *lfile;
 	int error;
 
-	lfile = malloc(sizeof(struct line_file));
-	if (lfile == NULL)
-		return pr_enomem();
+	lfile = pmalloc(sizeof(struct line_file));
 
 	lfile->file = fopen(file_name, "r");
 	if (lfile->file == NULL) {
@@ -74,10 +69,9 @@ lfile_read(struct line_file *lfile, char **result)
 	 * - If the file is empty, or all that's left is an empty line, it
 	 *   (confusingly) returns -1. errno will be 0, feof() should return
 	 *   1, ferror() should return 0.
-	 * - The fact that it returns the newline in the buffer is puzzling,
-	 *   because who the fuck wants that nonsense. You will want to remove
-	 *   it, BUT DON'T SWEAT IT IF IT'S NOT THERE, because the last line of
-	 *   the file might not be newline-terminated.
+	 * - It includes the newline in the result, which is puzzling. You will
+	 *   want to remove it, BUT DON'T SWEAT IT IF IT'S NOT THERE, because
+	 *   the last line of the file might not be newline-terminated.
 	 * - The string WILL be NULL-terminated, but the NULL chara will not be
 	 *   included in the returned length. BUT IT'S THERE. Don't worry about
 	 *   writing past the allocated space on the last line.
@@ -93,6 +87,9 @@ lfile_read(struct line_file *lfile, char **result)
 	 * because getline is normally meant to be used repeatedly with a
 	 * recycled buffer. (free() is a no-op if its argument is NULL so go
 	 * nuts.)
+	 *
+	 * Update: If you remove getline(), consider downgrading _POSIX_C_SOURCE
+	 * to 200112L. (And _XOPEN_SOURCE to 600.)
 	 */
 
 	string = NULL;
