@@ -1,28 +1,90 @@
+#include <errno.h>
+
 #include "config.h"
 #include "extension.h"
-#include "internal_pool.h"
+#include "log.h"
 #include "nid.h"
-#include "reqs_errors.h"
 #include "thread_var.h"
-#include "validation_run.h"
 #include "http/http.h"
+#include "incidence/incidence.h"
 #include "rtr/rtr.h"
 #include "rtr/db/vrps.h"
 #include "xml/relax_ng.h"
-#include "rrdp/db/db_rrdp.h"
 
 static int
-run_rtr_server(void)
+fort_standalone(void)
 {
 	int error;
 
+	pr_op_info("Updating cache...");
+
+	error = vrps_update(NULL);
+	if (error) {
+		pr_op_err("Validation unsuccessful; results unusable.");
+		return error;
+	}
+
+	pr_op_info("Done.");
+	return 0;
+}
+
+static int
+fort_server(void)
+{
+	int error;
+	bool changed;
+
+	pr_op_info("Main loop: Starting...");
+
 	error = rtr_start();
 	if (error)
 		return error;
 
-	error = validation_run_first();
-	if (!error)
-		error = validation_run_cycle(); /* Usually loops forever */
+	error = vrps_update(NULL);
+	if (error) {
+		pr_op_err("Main loop: Validation unsuccessful; results unusable.");
+		return error;
+	}
+
+	rtr_notify();
+
+	/*
+	 * According to some past experience I can't find anymore, there's at
+	 * least one brand of router that misunderstands RTR "No Data Available
+	 * [yet];" it thinks it's a fatal error.
+	 *
+	 * I don't know if the problem persists, I can't find the bug report
+	 * anymore, and given that I've patched several RTR errors since, I'm
+	 * questioning whether it was a problem in the first place.
+	 *
+	 * Tentatively tell the admin to start those routers now. This is
+	 * ridiculous on several levels however, and I'm half a mind to delete
+	 * this notification.
+	 *
+	 * This message was born in 38e256cb, and I've decided to downgrade and
+	 * reword it in the hopes of triggering complaints if someone's still
+	 * using it.
+	 *
+	 * TODO (fine) If nobody complaints, remove in a few months.
+	 */
+	pr_op_info("Main loop: Ready for routers.");
+
+	do {
+		pr_op_info("Main loop: Sleeping.");
+		sleep(config_get_validation_interval());
+		pr_op_info("Main loop: Time to work!");
+
+		error = vrps_update(&changed);
+		if (error == -EINTR)
+			break;
+		if (error) {
+			pr_op_debug("Main loop: Error %d (%s)", error,
+			    strerror(abs(error)));
+			continue;
+		}
+		if (changed)
+			rtr_notify();
+	} while (true);
 
 	rtr_stop();
 	return error;
@@ -91,48 +153,29 @@ main(int argc, char **argv)
 	if (error)
 		goto revert_nid;
 
-	/*
-	 * TODO (performance) this looks like a lot of overhead. Is it really
-	 * necessary when mode is STANDALONE?
-	 */
-	error = internal_pool_init();
-	if (error)
-		goto revert_http;
-
 	error = relax_ng_init();
 	if (error)
-		goto revert_pool;
+		goto revert_http;
 	error = vrps_init();
 	if (error)
 		goto revert_relax_ng;
-	error = db_rrdp_init();
-	if (error)
-		goto vrps_cleanup;
-	error = reqs_errors_init();
-	if (error)
-		goto db_rrdp_cleanup;
 
-	/* Do stuff */
+	/* Meat */
+
 	switch (config_get_mode()) {
 	case STANDALONE:
-		error = validation_run_first();
+		error = fort_standalone();
 		break;
 	case SERVER:
-		error = run_rtr_server();
+		error = fort_server();
 		break;
 	}
 
 	/* End */
 
-	reqs_errors_cleanup();
-db_rrdp_cleanup:
-	db_rrdp_cleanup();
-vrps_cleanup:
 	vrps_destroy();
 revert_relax_ng:
 	relax_ng_cleanup();
-revert_pool:
-	internal_pool_cleanup();
 revert_http:
 	http_cleanup();
 revert_nid:
