@@ -1,8 +1,6 @@
 #ifndef SRC_OBJECT_CERTIFICATE_H_
 #define SRC_OBJECT_CERTIFICATE_H_
 
-#include <stdbool.h>
-#include <openssl/x509.h>
 #include "certificate_refs.h"
 #include "resource.h"
 #include "rpp.h"
@@ -12,14 +10,12 @@
 
 /* Certificate types in the RPKI */
 enum cert_type {
-	TA,		/* Trust Anchor */
-	CA,		/* Certificate Authority */
-	BGPSEC,		/* BGPsec certificates */
-	EE,		/* End Entity certificates */
+	CERTYPE_TA,		/* Trust Anchor */
+	CERTYPE_CA,		/* Certificate Authority */
+	CERTYPE_BGPSEC,		/* BGPsec certificates */
+	CERTYPE_EE,		/* End Entity certificates */
 };
 
-int certificate_load(struct rpki_uri *, X509 **);
-
 /**
  * Performs the basic (RFC 5280, presumably) chain validation.
  * (Ignores the IP and AS extensions.)
@@ -52,6 +48,8 @@ int certificate_get_resources(X509 *, struct resources *, enum cert_type);
  */
 int certificate_validate_extensions_ee(X509 *, OCTET_STRING_t *,
     struct certificate_refs *, enum rpki_policy *);
+int certificate_validate_extensions_bgpsec(X509 *, unsigned char **,
+    enum rpki_policy *, struct rpp *);
 
 /*
  * Specific validation of AIA (rfc6487#section-4.8.7) extension, public so that
