@@ -2,7 +2,7 @@
 #define SRC_OBJECT_CRL_H_
 
 #include <openssl/x509.h>
-#include "uri.h"
+#include "types/uri.h"
 
 int crl_load(struct rpki_uri *uri, X509_CRL **);
 
