@@ -4,7 +4,7 @@
 #include "thread_var.h"
 #include "asn1/oid.h"
 #include "object/signed_object.h"
-#include "vcard.h"
+#include "object/vcard.h"
 
 static int
 handle_vcard(struct signed_object *sobj)
@@ -37,9 +37,7 @@ ghostbusters_traverse(struct rpki_uri *uri, struct rpp *pp)
 	error = rpp_crl(pp, &crl);
 	if (error)
 		goto revert_sobj;
-	error = signed_object_args_init(&sobj_args, uri, crl, true);
-	if (error)
-		goto revert_sobj;
+	signed_object_args_init(&sobj_args, uri, crl, true);
 
 	/* Validate everything */
 	error = signed_object_validate(&sobj, &arcs, &sobj_args);
