@@ -1,6 +1,4 @@
-#include "manifest.h"
-
-#include <errno.h>
+#include "object/manifest.h"
 
 #include "algorithm.h"
 #include "common.h"
@@ -16,6 +14,20 @@
 #include "object/roa.h"
 #include "object/signed_object.h"
 
+static int
+cage(struct rpki_uri **uri, struct rpki_uri *notif)
+{
+	if (notif == NULL) {
+		/* No need to cage */
+		uri_refget(*uri);
+		return 0;
+	}
+
+	return uri_create_caged(uri,
+	    tal_get_file_name(validation_tal(state_retrieve())), notif,
+	    uri_get_global(*uri), uri_get_global_len(*uri));
+}
+
 static int
 decode_manifest(struct signed_object *sobj, struct Manifest **result)
 {
@@ -28,6 +40,29 @@ decode_manifest(struct signed_object *sobj, struct Manifest **result)
 	);
 }
 
+/*
+ * Expects both arguments to be normalized and CST.
+ */
+static int
+tm_cmp(struct tm *tm1, struct tm *tm2)
+{
+#define TM_CMP(field)							\
+	if (tm1->field < tm2->field)					\
+		return -1;						\
+	if (tm1->field > tm2->field)					\
+		return 1;						\
+
+	TM_CMP(tm_year);
+	TM_CMP(tm_mon);
+	TM_CMP(tm_mday);
+	TM_CMP(tm_hour);
+	TM_CMP(tm_min);
+	TM_CMP(tm_sec);
+	return 0;
+
+#undef TM_CMP
+}
+
 static int
 validate_dates(GeneralizedTime_t *this, GeneralizedTime_t *next)
 {
@@ -36,44 +71,46 @@ validate_dates(GeneralizedTime_t *this, GeneralizedTime_t *next)
 	tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,			\
 	tm.tm_hour, tm.tm_min, tm.tm_sec
 
-	time_t thisUpdate;
-	time_t nextUpdate;
-	time_t now;
-	struct tm thisUpdate_tm;
-	struct tm nextUpdate_tm;
+	time_t now_tt;
+	struct tm now;
+	struct tm thisUpdate;
+	struct tm nextUpdate;
 	int error;
 
-	/*
-	 * BTW: We only need the tm variables for error messages, which are
-	 * rarely needed.
-	 * So maybe we could get a small performance boost by postponing the
-	 * calls to localtime_r().
-	 */
-	thisUpdate = asn_GT2time(this, &thisUpdate_tm, false);
-	nextUpdate = asn_GT2time(next, &nextUpdate_tm, false);
+	error = asn_GT2time(this, &thisUpdate);
+	if (error)
+		return pr_val_err("Manifest's thisUpdate date is unparseable.");
+	error = asn_GT2time(next, &nextUpdate);
+	if (error)
+		return pr_val_err("Manifest's nextUpdate date is unparseable.");
 
-	if (difftime(thisUpdate, nextUpdate) > 0) {
+	if (tm_cmp(&thisUpdate, &nextUpdate) > 0) {
 		return pr_val_err(
 		    "Manifest's thisUpdate (" TM_FMT ") > nextUpdate ("
 		        TM_FMT ").",
-		    TM_ARGS(thisUpdate_tm),
-		    TM_ARGS(nextUpdate_tm));
+		    TM_ARGS(thisUpdate),
+		    TM_ARGS(nextUpdate));
 	}
 
-	now = 0;
-	error = get_current_time(&now);
+	now_tt = 0;
+	error = get_current_time(&now_tt);
 	if (error)
 		return error;
+	if (gmtime_r(&now_tt, &now) == NULL) {
+		error = errno;
+		return pr_val_err("gmtime_r(now) error %d: %s", error,
+		    strerror(error));
+	}
 
-	if (difftime(now, thisUpdate) < 0) {
+	if (tm_cmp(&now, &thisUpdate) < 0) {
 		return pr_val_err(
 		    "Manifest is not valid yet. (thisUpdate: " TM_FMT ")",
-		    TM_ARGS(thisUpdate_tm));
+		    TM_ARGS(thisUpdate));
 	}
-	if (difftime(now, nextUpdate) > 0) {
+	if (tm_cmp(&now, &nextUpdate) > 0) {
 		return incidence(INID_MFT_STALE,
 		    "Manifest is stale. (nextUpdate: " TM_FMT ")",
-		    TM_ARGS(nextUpdate_tm));
+		    TM_ARGS(nextUpdate));
 	}
 
 	return 0;
@@ -152,25 +189,23 @@ validate_manifest(struct Manifest *manifest)
 }
 
 static int
-build_rpp(struct Manifest *mft, struct rpki_uri *mft_uri, bool rrdp_workspace,
-    struct rpp **pp)
+build_rpp(struct Manifest *mft, struct rpki_uri *notif,
+    struct rpki_uri *mft_uri, struct rpp **pp)
 {
+	char const *tal;
 	int i;
 	struct FileAndHash *fah;
 	struct rpki_uri *uri;
 	int error;
 
 	*pp = rpp_create();
-	if (*pp == NULL)
-		return pr_enomem();
+
+	tal = tal_get_file_name(validation_tal(state_retrieve()));
 
 	for (i = 0; i < mft->fileList.list.count; i++) {
 		fah = mft->fileList.list.array[i];
 
-		error = uri_create_mft(&uri, mft_uri, &fah->file,
-		    rrdp_workspace);
-		if (error == ESKIP)
-			continue;
+		error = uri_create_mft(&uri, tal, notif, mft_uri, &fah->file);
 		/*
 		 * Not handling ENOTRSYNC is fine because the manifest URL
 		 * should have been RSYNC. Something went wrong if an RSYNC URL
@@ -189,7 +224,7 @@ build_rpp(struct Manifest *mft, struct rpki_uri *mft_uri, bool rrdp_workspace,
 		 * - Positive value: file doesn't exist and keep validating
 		 *   manifest.
 		 */
-		error = hash_validate_mft_file("sha256", uri, &fah->hash);
+		error = hash_validate_mft_file(uri, &fah->hash);
 		if (error < 0) {
 			uri_refput(uri);
 			goto fail;
@@ -200,13 +235,13 @@ build_rpp(struct Manifest *mft, struct rpki_uri *mft_uri, bool rrdp_workspace,
 		}
 
 		if (uri_has_extension(uri, ".cer"))
-			error = rpp_add_cert(*pp, uri);
+			rpp_add_cert(*pp, uri);
 		else if (uri_has_extension(uri, ".roa"))
-			error = rpp_add_roa(*pp, uri);
+			rpp_add_roa(*pp, uri);
 		else if (uri_has_extension(uri, ".crl"))
 			error = rpp_add_crl(*pp, uri);
 		else if (uri_has_extension(uri, ".gbr"))
-			error = rpp_add_ghostbusters(*pp, uri);
+			rpp_add_ghostbusters(*pp, uri);
 		else
 			uri_refput(uri); /* ignore it. */
 
@@ -234,7 +269,7 @@ fail:
  * @pp. If @rrdp_workspace is true, use the local RRDP repository.
  */
 int
-handle_manifest(struct rpki_uri *uri, bool rrdp_workspace, struct rpp **pp)
+handle_manifest(struct rpki_uri *uri, struct rpki_uri *notif, struct rpp **pp)
 {
 	static OID oid = OID_MANIFEST;
 	struct oid_arcs arcs = OID2ARCS("manifest", oid);
@@ -245,6 +280,9 @@ handle_manifest(struct rpki_uri *uri, bool rrdp_workspace, struct rpp **pp)
 	int error;
 
 	/* Prepare */
+	error = cage(&uri, notif); /* ref++ */
+	if (error)
+		return error;
 	pr_val_debug("Manifest '%s' {", uri_val_get_printable(uri));
 	fnstack_push_uri(uri);
 
@@ -257,7 +295,7 @@ handle_manifest(struct rpki_uri *uri, bool rrdp_workspace, struct rpp **pp)
 		goto revert_sobj;
 
 	/* Initialize out parameter (@pp) */
-	error = build_rpp(mft, uri, rrdp_workspace, pp);
+	error = build_rpp(mft, notif, uri, pp);
 	if (error)
 		goto revert_manifest;
 
@@ -265,9 +303,7 @@ handle_manifest(struct rpki_uri *uri, bool rrdp_workspace, struct rpp **pp)
 	error = rpp_crl(*pp, &crl);
 	if (error)
 		goto revert_rpp;
-	error = signed_object_args_init(&sobj_args, uri, crl, false);
-	if (error)
-		goto revert_rpp;
+	signed_object_args_init(&sobj_args, uri, crl, false);
 
 	/* Validate everything */
 	error = signed_object_validate(&sobj, &arcs, &sobj_args);
@@ -295,5 +331,6 @@ revert_sobj:
 revert_log:
 	pr_val_debug("}");
 	fnstack_pop();
+	uri_refput(uri); /* ref-- */
 	return error;
 }
