@@ -1,13 +1,13 @@
 #include "rtr/db/db_table.h"
 
-#include <sys/types.h> /* AF_INET, AF_INET6 (needed in OpenBSD) */
-#include <sys/socket.h> /* AF_INET, AF_INET6 (needed in OpenBSD) */
+#include <errno.h>
 
+#include "alloc.h"
 #include "log.h"
-#include "data_structure/uthash_nonfatal.h"
+#include "data_structure/uthash.h"
 
 struct hashable_roa {
-	const struct vrp data;
+	struct vrp data;
 	UT_hash_handle hh;
 };
 
@@ -26,79 +26,41 @@ db_table_create(void)
 {
 	struct db_table *table;
 
-	table = malloc(sizeof(struct db_table));
-	if (table == NULL)
-		return NULL;
-
+	table = pmalloc(sizeof(struct db_table));
 	table->roas = NULL;
 	table->router_keys = NULL;
+
 	return table;
 }
 
 void
 db_table_destroy(struct db_table *table)
 {
-	struct hashable_roa *node;
-	struct hashable_roa *tmp;
-	struct hashable_key *node_key;
-	struct hashable_key *tmp_key;
+	struct hashable_roa *roa, *tmpr;
+	struct hashable_key *rk, *tmpk;
 
-	HASH_ITER(hh, table->roas, node, tmp) {
-		HASH_DEL(table->roas, node);
-		free(node);
+	if (table == NULL)
+		return;
+
+	HASH_ITER(hh, table->roas, roa, tmpr) {
+		HASH_DEL(table->roas, roa);
+		free(roa);
 	}
 
-	HASH_ITER(hh, table->router_keys, node_key, tmp_key) {
-		HASH_DEL(table->router_keys, node_key);
-		free(node_key);
+	HASH_ITER(hh, table->router_keys, rk, tmpk) {
+		HASH_DEL(table->router_keys, rk);
+		free(rk);
 	}
 
 	free(table);
 }
 
-int
-db_table_foreach_roa(struct db_table const *table, vrp_foreach_cb cb, void *arg)
-{
-	struct hashable_roa *node, *tmp;
-	int error;
-
-	HASH_ITER(hh, table->roas, node, tmp) {
-		error = cb(&node->data, arg);
-		if (error)
-			return error;
-	}
-
-	return 0;
-}
-
-int
-db_table_foreach_router_key(struct db_table const *table,
-    router_key_foreach_cb cb, void *arg)
-{
-	struct hashable_key *node, *tmp;
-	int error;
-
-	HASH_ITER(hh, table->router_keys, node, tmp) {
-		error = cb(&node->data, arg);
-		if (error)
-			return error;
-	}
-
-	return 0;
-}
-
 static int
-add_roa(struct db_table *table, struct hashable_roa const *stack_new)
+add_roa(struct db_table *table, struct hashable_roa *new)
 {
-	struct hashable_roa *new;
 	struct hashable_roa *old;
 	int error;
 
-	new = malloc(sizeof(struct hashable_roa));
-	if (new == NULL)
-		return pr_enomem();
-	memcpy(new, stack_new, sizeof(*new));
-
 	errno = 0;
 	HASH_REPLACE(hh, table->roas, data, sizeof(new->data), new, old);
 	error = errno;
@@ -133,6 +95,62 @@ add_router_key(struct db_table *table, struct hashable_key *new)
 	return 0;
 }
 
+/* Moves the content from @src into @dst. */
+int
+db_table_join(struct db_table *dst, struct db_table *src)
+{
+	struct hashable_roa *roa, *tmpr;
+	struct hashable_key *rk, *tmpk;
+	int error;
+
+	HASH_ITER(hh, src->roas, roa, tmpr) {
+		HASH_DEL(src->roas, roa);
+		error = add_roa(dst, roa);
+		if (error)
+			return error;
+	}
+
+	HASH_ITER(hh, src->router_keys, rk, tmpk) {
+		HASH_DEL(src->router_keys, rk);
+		error = add_router_key(dst, rk);
+		if (error)
+			return error;
+	}
+
+	return 0;
+}
+
+int
+db_table_foreach_roa(struct db_table const *table, vrp_foreach_cb cb, void *arg)
+{
+	struct hashable_roa *node, *tmp;
+	int error;
+
+	HASH_ITER(hh, table->roas, node, tmp) {
+		error = cb(&node->data, arg);
+		if (error)
+			return error;
+	}
+
+	return 0;
+}
+
+int
+db_table_foreach_router_key(struct db_table const *table,
+    router_key_foreach_cb cb, void *arg)
+{
+	struct hashable_key *node, *tmp;
+	int error;
+
+	HASH_ITER(hh, table->router_keys, node, tmp) {
+		error = cb(&node->data, arg);
+		if (error)
+			return error;
+	}
+
+	return 0;
+}
+
 unsigned int
 db_table_roa_count(struct db_table *table)
 {
@@ -174,44 +192,40 @@ int
 rtrhandler_handle_roa_v4(struct db_table *table, uint32_t asn,
     struct ipv4_prefix const *prefix4, uint8_t max_length)
 {
-	struct hashable_roa new = {
-		.data.asn = asn,
-		.data.prefix.v4 = prefix4->addr,
-		.data.prefix_length = prefix4->len,
-		.data.max_prefix_length = max_length,
-		.data.addr_fam = AF_INET,
-	};
+	struct hashable_roa *roa = pzalloc(sizeof(struct hashable_roa));
 
-	return add_roa(table, &new);
+	roa->data.asn = asn;
+	roa->data.prefix.v4 = prefix4->addr;
+	roa->data.prefix_length = prefix4->len;
+	roa->data.max_prefix_length = max_length;
+	roa->data.addr_fam = AF_INET;
+
+	return add_roa(table, roa);
 }
 
 int
 rtrhandler_handle_roa_v6(struct db_table *table, uint32_t asn,
     struct ipv6_prefix const *prefix6, uint8_t max_length)
 {
-	struct hashable_roa new = {
-		.data.asn = asn,
-		.data.prefix.v6 = prefix6->addr,
-		.data.prefix_length = prefix6->len,
-		.data.max_prefix_length = max_length,
-		.data.addr_fam = AF_INET6,
-	};
+	struct hashable_roa *roa = pzalloc(sizeof(struct hashable_roa));
 
-	return add_roa(table, &new);
+	roa->data.asn = asn;
+	roa->data.prefix.v6 = prefix6->addr;
+	roa->data.prefix_length = prefix6->len;
+	roa->data.max_prefix_length = max_length;
+	roa->data.addr_fam = AF_INET6;
+
+	return add_roa(table, roa);
 }
 
 int
-rtrhandler_handle_router_key(struct db_table *table,
-    unsigned char const *ski, uint32_t as, unsigned char const *spk)
+rtrhandler_handle_router_key(struct db_table *table, unsigned char const *ski,
+    uint32_t as, unsigned char const *spk)
 {
 	struct hashable_key *key;
 	int error;
 
-	key = malloc(sizeof(struct hashable_key));
-	if (key == NULL)
-		return pr_enomem();
-	/* Needed by uthash */
-	memset(key, 0, sizeof(struct hashable_key));
+	key = pzalloc(sizeof(struct hashable_key)); /* Zero needed by uthash */
 
 	router_key_init(&key->data, ski, as, spk);
 
@@ -236,63 +250,25 @@ add_roa_deltas(struct hashable_roa *roas1, struct hashable_roa *roas2,
 	struct hashable_roa *tmp;
 	unsigned int r;
 	unsigned int roa1_count;
-	int error;
 
 	r = 0;
 	roa1_count = HASH_COUNT(roas1);
 
 	HASH_ITER(hh, roas1, n1, tmp) {
 		HASH_FIND(hh, roas2, &n1->data, sizeof(n1->data), n2);
-		if (n2 == NULL) {
-			error = deltas_add_roa(deltas, &n1->data, op,
-			    r1type, r, roa1_count);
-			if (error)
-				return error;
-		}
+		if (n2 == NULL)
+			deltas_add_roa(deltas, &n1->data, op, r1type, r,
+			    roa1_count);
 		r++;
 	}
 
 	return 0;
 }
 
-void
-find_bad_vrp(char const *prefix, struct db_table *table)
-{
-	struct hashable_roa *node;
-	struct hashable_roa *tmp;
-	struct vrp const *vrp;
-	unsigned int roa_counter;
-	unsigned int roa_count;
-	char buffer[INET6_ADDRSTRLEN];
-
-	if (table == NULL)
-		return;
-
-	roa_counter = 0;
-	roa_count = HASH_COUNT(table->roas);
-
-	HASH_ITER(hh, table->roas, node, tmp) {
-		vrp = &node->data;
-		if (vrp->addr_fam != AF_INET && vrp->addr_fam != AF_INET6) {
-			pr_crit("%s: VRP corrupted! [%u %s/%u-%u %u] %u/%u "
-			    "(Please report this output to https://github.com/NICMx/FORT-validator/issues/89)",
-			    prefix,
-			    vrp->asn,
-			    addr2str6(&vrp->prefix.v6, buffer),
-			    vrp->prefix_length,
-			    vrp->max_prefix_length,
-			    vrp->addr_fam,
-			    roa_counter,
-			    roa_count);
-		}
-		roa_counter++;
-	}
-}
-
-static int
+static void
 add_router_key_delta(struct deltas *deltas, struct hashable_key *key, int op)
 {
-	return deltas_add_router_key(deltas, &key->data, op);
+	deltas_add_router_key(deltas, &key->data, op);
 }
 
 /*
@@ -300,58 +276,36 @@ add_router_key_delta(struct deltas *deltas, struct hashable_key *key, int op)
  *
  * (Places the Router Keys that exist in @keys1 but not in @key2 in @deltas.)
  */
-static int
+static void
 add_router_key_deltas(struct hashable_key *keys1, struct hashable_key *keys2,
     struct deltas *deltas, int op)
 {
 	struct hashable_key *n1; /* A node from @keys1 */
 	struct hashable_key *n2; /* A node from @keys2 */
-	int error;
 
 	for (n1 = keys1; n1 != NULL; n1 = n1->hh.next) {
 		HASH_FIND(hh, keys2, &n1->data, sizeof(n1->data), n2);
-		if (n2 == NULL) {
-			error = add_router_key_delta(deltas, n1, op);
-			if (error)
-				return error;
-		}
+		if (n2 == NULL)
+			add_router_key_delta(deltas, n1, op);
+	}
+}
+
+struct deltas *
+compute_deltas(struct db_table *old, struct db_table *new)
+{
+	struct deltas *deltas = deltas_create();
+
+	add_roa_deltas(new->roas, old->roas, deltas, FLAG_ANNOUNCEMENT, 'n');
+	add_roa_deltas(old->roas, new->roas, deltas, FLAG_WITHDRAWAL, 'o');
+	add_router_key_deltas(new->router_keys, old->router_keys, deltas,
+	    FLAG_ANNOUNCEMENT);
+	add_router_key_deltas(old->router_keys, new->router_keys, deltas,
+	    FLAG_WITHDRAWAL);
+
+	if (deltas_is_empty(deltas)) {
+		deltas_refput(deltas);
+		return NULL;
 	}
 
-	return 0;
-}
-
-int
-compute_deltas(struct db_table *old, struct db_table *new,
-    struct deltas **result)
-{
-	struct deltas *deltas;
-	int error;
-
-	error = deltas_create(&deltas);
-	if (error)
-		return error;
-
-	error = add_roa_deltas(new->roas, old->roas, deltas, FLAG_ANNOUNCEMENT,
-	    'n');
-	if (error)
-		goto fail;
-	error = add_roa_deltas(old->roas, new->roas, deltas, FLAG_WITHDRAWAL,
-	    'o');
-	if (error)
-		goto fail;
-	error = add_router_key_deltas(new->router_keys, old->router_keys,
-	    deltas, FLAG_ANNOUNCEMENT);
-	if (error)
-		goto fail;
-	error = add_router_key_deltas(old->router_keys, new->router_keys,
-	    deltas, FLAG_WITHDRAWAL);
-	if (error)
-		goto fail;
-
-	*result = deltas;
-	return 0;
-
-fail:
-	deltas_refput(deltas);
-	return error;
+	return deltas;
 }
