@@ -1,26 +1,10 @@
-#include "pdu_handler.h"
+#include "rtr/pdu_handler.h"
 
 #include <errno.h>
-#include <stddef.h>
-#include <unistd.h>
-
-#include "err_pdu.h"
 #include "log.h"
-#include "pdu.h"
-#include "pdu_sender.h"
-#include "rtr/db/vrps.h"
-
-#define WARN_UNEXPECTED_PDU(name, fd, request, pdu_name)		\
-	struct name##_pdu *pdu = request->pdu;				\
-	return err_pdu_send_invalid_request(fd,				\
-	    pdu->header.protocol_version,				\
-	    request, "Clients are not supposed to send " pdu_name " PDUs.");
-
-int
-handle_serial_notify_pdu(int fd, struct rtr_request const *request)
-{
-	WARN_UNEXPECTED_PDU(serial_notify, fd, request, "Serial Notify");
-}
+#include "rtr/err_pdu.h"
+#include "rtr/pdu_sender.h"
+#include "rtr/pdu_stream.h"
 
 struct send_delta_args {
 	int fd;
@@ -72,13 +56,21 @@ send_delta_rk(struct delta_router_key const *delta, void *arg)
 }
 
 int
-handle_serial_query_pdu(int fd, struct rtr_request const *request)
+handle_serial_query_pdu(struct rtr_request *request)
 {
-	struct serial_query_pdu *query = request->pdu;
 	struct send_delta_args args;
 	serial_t final_serial;
 	int error;
 
+	pr_op_debug("Serial Query. Request version/session/serial: %u/%u/%u",
+	    request->pdu.rtr_version,
+	    request->pdu.obj.sq.session_id,
+	    request->pdu.obj.sq.serial_number);
+
+	args.fd = request->fd;
+	args.rtr_version = request->pdu.rtr_version;
+	args.cache_response_sent = false;
+
 	/*
 	 * RFC 6810 and 8210:
 	 * "If [...] either the router or the cache finds that the value of the
@@ -86,14 +78,9 @@ handle_serial_query_pdu(int fd, struct rtr_request const *request)
 	 * the mismatch MUST immediately terminate the session with an Error
 	 * Report PDU with code 0 ("Corrupt Data")"
 	 */
-	args.rtr_version = query->header.protocol_version;
-	if (query->header.m.session_id !=
-	    get_current_session_id(args.rtr_version))
-		return err_pdu_send_corrupt_data(fd, args.rtr_version, request,
-		    "Session ID doesn't match.");
-
-	args.fd = fd;
-	args.cache_response_sent = false;
+	if (request->pdu.obj.sq.session_id != get_current_session_id(args.rtr_version))
+		return err_pdu_send_corrupt_data(args.fd, args.rtr_version,
+			&request->pdu.raw, "Session ID doesn't match.");
 
 	/*
 	 * For the record, there are two reasons why we want to work on a
@@ -104,8 +91,8 @@ handle_serial_query_pdu(int fd, struct rtr_request const *request)
 	 *    PDUs, to minimize writer stagnation.
 	 */
 
-	error = vrps_foreach_delta_since(query->serial_number, &final_serial,
-	    send_delta_vrp, send_delta_rk, &args);
+	error = vrps_foreach_delta_since(request->pdu.obj.sq.serial_number,
+	    &final_serial, send_delta_vrp, send_delta_rk, &args);
 	switch (error) {
 	case 0:
 		/*
@@ -115,19 +102,20 @@ handle_serial_query_pdu(int fd, struct rtr_request const *request)
 		 * and programming errors. Best avoid error PDUs.
 		 */
 		if (!args.cache_response_sent) {
-			error = send_cache_response_pdu(fd, args.rtr_version);
+			error = send_cache_response_pdu(args.fd,
+			    args.rtr_version);
 			if (error)
 				return error;
 		}
-		return send_end_of_data_pdu(fd, args.rtr_version, final_serial);
+		return send_end_of_data_pdu(args.fd, args.rtr_version,
+		    final_serial);
 	case -EAGAIN: /* Database still under construction */
-		error = err_pdu_send_no_data_available(fd, args.rtr_version);
-		return error;
+		return err_pdu_send_no_data_available(args.fd, args.rtr_version);
 	case -ESRCH: /* Invalid serial */
 		/* https://tools.ietf.org/html/rfc6810#section-6.3 */
-		return send_cache_reset_pdu(fd, args.rtr_version);
+		return send_cache_reset_pdu(args.fd, args.rtr_version);
 	case -ENOMEM: /* Memory allocation failure */
-		return pr_enomem();
+		enomem_panic();
 	case EAGAIN: /* Too many threads */
 		/*
 		 * I think this should be more of a "try again" thing, but
@@ -136,7 +124,7 @@ handle_serial_query_pdu(int fd, struct rtr_request const *request)
 		break;
 	}
 
-	return err_pdu_send_internal_error(fd, args.rtr_version);
+	return err_pdu_send_internal_error(args.fd, args.rtr_version);
 }
 
 struct base_roa_args {
@@ -161,8 +149,6 @@ send_base_roa(struct vrp const *vrp, void *arg)
 	return send_prefix_pdu(args->fd, args->version, vrp, FLAG_ANNOUNCEMENT);
 }
 
-
-
 static int
 send_base_router_key(struct router_key const *key, void *arg)
 {
@@ -181,25 +167,24 @@ send_base_router_key(struct router_key const *key, void *arg)
 }
 
 int
-handle_reset_query_pdu(int fd, struct rtr_request const *request)
+handle_reset_query_pdu(struct rtr_request *request)
 {
-	struct reset_query_pdu *pdu = request->pdu;
 	struct base_roa_args args;
 	serial_t current_serial;
 	int error;
 
 	args.started = false;
-	args.fd = fd;
-	args.version = pdu->header.protocol_version;
+	args.fd = request->fd;
+	args.version = request->pdu.rtr_version;
 
 	error = get_last_serial_number(&current_serial);
 	switch (error) {
 	case 0:
 		break;
 	case -EAGAIN:
-		return err_pdu_send_no_data_available(fd, args.version);
+		return err_pdu_send_no_data_available(args.fd, args.version);
 	default:
-		err_pdu_send_internal_error(fd, args.version);
+		err_pdu_send_internal_error(args.fd, args.version);
 		return error;
 	}
 
@@ -218,73 +203,19 @@ handle_reset_query_pdu(int fd, struct rtr_request const *request)
 		/* Assure that cache response is (or was) sent */
 		if (args.started)
 			break;
-		error = send_cache_response_pdu(fd, args.version);
+		error = send_cache_response_pdu(args.fd, args.version);
 		if (error)
 			return error;
 		break;
 	case -EAGAIN:
-		return err_pdu_send_no_data_available(fd, args.version);
+		return err_pdu_send_no_data_available(args.fd, args.version);
 	case EAGAIN:
-		err_pdu_send_internal_error(fd, args.version);
+		err_pdu_send_internal_error(args.fd, args.version);
 		return error;
 	default:
 		/* Any other error must stop sending more PDUs */
 		return error;
 	}
 
-	return send_end_of_data_pdu(fd, args.version, current_serial);
-}
-
-int
-handle_cache_response_pdu(int fd, struct rtr_request const *request)
-{
-	WARN_UNEXPECTED_PDU(cache_response, fd, request, "Cache Response");
-}
-
-int
-handle_ipv4_prefix_pdu(int fd, struct rtr_request const *request)
-{
-	WARN_UNEXPECTED_PDU(ipv4_prefix, fd, request, "IPv4 Prefix");
-}
-
-int
-handle_ipv6_prefix_pdu(int fd, struct rtr_request const *request)
-{
-	WARN_UNEXPECTED_PDU(ipv6_prefix, fd, request, "IPv6 Prefix");
-}
-
-int
-handle_end_of_data_pdu(int fd, struct rtr_request const *request)
-{
-	WARN_UNEXPECTED_PDU(end_of_data, fd, request, "End of Data");
-}
-
-int
-handle_cache_reset_pdu(int fd, struct rtr_request const *request)
-{
-	WARN_UNEXPECTED_PDU(cache_reset, fd, request, "Cache Reset");
-}
-
-int
-handle_router_key_pdu(int fd, struct rtr_request const *request)
-{
-	WARN_UNEXPECTED_PDU(router_key, fd, request, "Router Key");
-}
-
-int
-handle_error_report_pdu(int fd, struct rtr_request const *request)
-{
-	struct error_report_pdu *received = request->pdu;
-	char const *error_name;
-
-	error_name = err_pdu_to_string(received->header.m.error_code);
-
-	if (received->error_message != NULL)
-		pr_op_info("Client responded with error PDU '%s' ('%s'). Closing socket.",
-		    error_name, received->error_message);
-	else
-		pr_op_info("Client responded with error PDU '%s'. Closing socket.",
-		    error_name);
-
-	return -EINVAL;
+	return send_end_of_data_pdu(args.fd, args.version, current_serial);
 }
