@@ -1,7 +1,7 @@
 #ifndef SRC_RTR_PDU_SENDER_H_
 #define SRC_RTR_PDU_SENDER_H_
 
-#include "pdu.h"
+#include "rtr/pdu.h"
 #include "types/router_key.h"
 #include "rtr/db/vrps.h"
 
@@ -11,8 +11,7 @@ int send_cache_response_pdu(int, uint8_t);
 int send_prefix_pdu(int, uint8_t, struct vrp const *, uint8_t);
 int send_router_key_pdu(int, uint8_t, struct router_key const *, uint8_t);
 int send_end_of_data_pdu(int, uint8_t, serial_t);
-int send_error_report_pdu(int, uint8_t, uint16_t, struct rtr_request const *,
+int send_error_report_pdu(int, uint8_t, uint16_t, struct rtr_buffer const *,
     char *);
 
-
 #endif /* SRC_RTR_PDU_SENDER_H_ */
