@@ -1,12 +1,8 @@
 #ifndef SRC_STATE_H_
 #define SRC_STATE_H_
 
-#include <openssl/x509.h>
-#include "cert_stack.h"
 #include "validation_handler.h"
 #include "object/tal.h"
-#include "rsync/rsync.h"
-#include "rrdp/db/db_rrdp_uris.h"
 
 struct validation;
 
@@ -15,9 +11,9 @@ int validation_prepare(struct validation **, struct tal *,
 void validation_destroy(struct validation *);
 
 struct tal *validation_tal(struct validation *);
+struct rpki_cache *validation_cache(struct validation *);
 X509_STORE *validation_store(struct validation *);
 struct cert_stack *validation_certstack(struct validation *);
-struct uri_list *validation_rsync_visited_uris(struct validation *);
 
 enum pubkey_state {
 	PKS_VALID,
@@ -36,6 +32,5 @@ struct validation_handler const *
 validation_get_validation_handler(struct validation *);
 
 struct db_rrdp_uri *validation_get_rrdp_uris(struct validation *);
-char const *validation_get_rrdp_workspace(struct validation *);
 
 #endif /* SRC_STATE_H_ */
