@@ -1,18 +1,13 @@
 #include "thread_var.h"
 
-#include <errno.h>
 #include <pthread.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <arpa/inet.h>
-#include <sys/socket.h>
 
+#include "alloc.h"
 #include "config.h"
+#include "log.h"
 
 static pthread_key_t state_key;
 static pthread_key_t filenames_key;
-static pthread_key_t repository_key;
 
 struct filename_stack {
 	/* This can be NULL. Abort all operations if this is the case. */
@@ -21,11 +16,6 @@ struct filename_stack {
 	unsigned int size;
 };
 
-struct working_repo {
-	char const *uri;
-	unsigned int level;
-};
-
 static void
 fnstack_discard(void *arg)
 {
@@ -34,13 +24,6 @@ fnstack_discard(void *arg)
 	free(files);
 }
 
-static void
-working_repo_discard(void *arg)
-{
-	struct working_repo *repo = arg;
-	free(repo);
-}
-
 /** Initializes this entire module. Call once per runtime lifetime. */
 int
 thvar_init(void)
@@ -69,14 +52,6 @@ thvar_init(void)
 		return error;
 	}
 
-	error = pthread_key_create(&repository_key, working_repo_discard);
-	if (error) {
-		pr_op_err(
-		    "Fatal: Errcode %d while initializing the 'working repository' thread variable.",
-		    error);
-		return error;
-	}
-
 	return 0;
 }
 
@@ -93,17 +68,14 @@ state_store(struct validation *state)
 	return error;
 }
 
-/* Returns the current thread's validation state. */
+/*
+ * Returns the current thread's validation state. Should not be used outside of
+ * validation threads.
+ */
 struct validation *
 state_retrieve(void)
 {
-	struct validation *state;
-
-	state = pthread_getspecific(state_key);
-	if (state == NULL)
-		pr_crit("Programming error: This thread lacks a validation state.");
-
-	return state;
+	return pthread_getspecific(state_key);
 }
 
 /** Initializes the current thread's fnstack. Call once per thread. */
@@ -113,16 +85,9 @@ fnstack_init(void)
 	struct filename_stack *files;
 	int error;
 
-	files = malloc(sizeof(struct filename_stack));
-	if (files == NULL)
-		return;
-
-	files->filenames = malloc(32 * sizeof(char *));
-	if (files->filenames == NULL) {
-		free(files);
-		return;
-	}
+	files = pmalloc(sizeof(struct filename_stack));
 
+	files->filenames = pmalloc(32 * sizeof(char *));
 	files->len = 0;
 	files->size = 32;
 
@@ -178,22 +143,14 @@ void
 fnstack_push(char const *file)
 {
 	struct filename_stack *files;
-	char const **tmp;
 
 	files = pthread_getspecific(filenames_key);
 	if (files == NULL || files->filenames == NULL)
 		return;
 
 	if (files->len >= files->size) {
-		tmp = realloc(files->filenames, 2 * files->size * sizeof(char *));
-		if (tmp == NULL) {
-			/* Oh noes */
-			free(files->filenames);
-			files->filenames = NULL;
-			return;
-		}
-
-		files->filenames = tmp;
+		files->filenames = prealloc(files->filenames,
+		    2 * files->size * sizeof(char *));
 		files->size *= 2;
 	}
 
@@ -238,122 +195,10 @@ fnstack_pop(void)
 	files->len--;
 }
 
-/** Initializes the current thread's working repo. Call once per thread. */
-void
-working_repo_init(void)
-{
-	struct working_repo *repo;
-	int error;
-
-	repo = malloc(sizeof(struct working_repo));
-	if (repo == NULL)
-		return;
-
-	repo->uri = NULL;
-	repo->level = 0;
-
-	error = pthread_setspecific(repository_key, repo);
-	if (error)
-		pr_op_err("pthread_setspecific() returned %d.", error);
-}
-
-void
-working_repo_cleanup(void)
-{
-	struct working_repo *repo;
-	int error;
-
-	repo = pthread_getspecific(repository_key);
-	if (repo == NULL)
-		return;
-
-	working_repo_discard(repo);
-
-	error = pthread_setspecific(repository_key, NULL);
-	if (error)
-		pr_op_err("pthread_setspecific() returned %d.", error);
-}
-
-/*
- * Call whenever a certificate has more than one repository where its childs
- * live (rsync or RRDP).
- */
-void
-working_repo_push(char const *location)
-{
-	struct working_repo *repo;
-
-	repo = pthread_getspecific(repository_key);
-	if (repo == NULL)
-		return;
-
-	repo->uri = location;
-}
-
-/*
- * Set the current repository level, must be called before trying to fetch the
- * repository.
- *
- * The level "calculation" must be done by the caller.
- */
-void
-working_repo_push_level(unsigned int level)
-{
-	struct working_repo *repo;
-
-	repo = pthread_getspecific(repository_key);
-	if (repo == NULL)
-		return;
-
-	repo->level = level;
-}
-
-char const *
-working_repo_peek(void)
-{
-	struct working_repo *repo;
-
-	repo = pthread_getspecific(repository_key);
-
-	return repo == NULL ? NULL : repo->uri;
-}
-
-unsigned int
-working_repo_peek_level(void)
-{
-	struct working_repo *repo;
-
-	repo = pthread_getspecific(repository_key);
-
-	return repo->level;
-}
-
-/*
- * Call once the certificate's repositories were downloaded (either successful
- * or erroneously).
- */
-void
-working_repo_pop(void)
-{
-	struct working_repo *repo;
-
-	repo = pthread_getspecific(repository_key);
-	if (repo == NULL)
-		return;
-
-	repo->uri = NULL;
-	repo->level = 0;
-}
-
 static char const *
 addr2str(int af, void const *addr, char *(*buffer_cb)(struct validation *))
 {
-	struct validation *state;
-
-	state = state_retrieve();
-	if (state == NULL)
-		return NULL;
-
+	struct validation *state = state_retrieve();
 	return inet_ntop(af, addr, buffer_cb(state), INET6_ADDRSTRLEN);
 }
 
