@@ -1,8 +1,6 @@
 #include "types/address.h"
 
-#include <stdlib.h> /* strtoul() */
-#include <string.h> /* memset(), memcpy() */
-#include <arpa/inet.h> /* inet_ntop(), htonl() */
+#include <errno.h>
 
 #include "log.h"
 #include "thread_var.h"
@@ -132,6 +130,17 @@ ipv6_suffix_mask(unsigned int prefix_len, struct in6_addr *result)
 	}
 }
 
+bool
+addr6_equals(struct in6_addr const *a, struct in6_addr const *b)
+{
+	unsigned int i;
+	for (i = 0; i < INET_ADDRSTRLEN; i++) {
+		if (a->s6_addr[i] != b->s6_addr[i])
+			return false;
+	}
+	return true;
+}
+
 bool
 prefix4_equals(struct ipv4_prefix const *a, struct ipv4_prefix const *b)
 {
@@ -141,7 +150,7 @@ prefix4_equals(struct ipv4_prefix const *a, struct ipv4_prefix const *b)
 bool
 prefix6_equals(struct ipv6_prefix const *a, struct ipv6_prefix const *b)
 {
-	return (a->len == b->len) && IN6_ARE_ADDR_EQUAL(&a->addr, &b->addr);
+	return (a->len == b->len) && addr6_equals(&a->addr, &b->addr);
 }
 
 /**
@@ -548,4 +557,3 @@ sockaddr2str(struct sockaddr_storage *sockaddr, char *buffer)
 	if (str == NULL)
 		strcpy(buffer, "(unprintable address)");
 }
-
