@@ -3,34 +3,30 @@
 #include <unistd.h>
 #include <sys/queue.h>
 
-#include "algorithm.c"
+#include "alloc.c"
 #include "common.c"
-#include "file.c"
-#include "impersonator.c"
-#include "json_parser.c"
-#include "log.c"
-#include "output_printer.c"
-#include "crypto/base64.c"
+#include "mock.c"
 #include "types/delta.c"
 #include "types/router_key.c"
 #include "types/serial.c"
 #include "types/vrp.c"
-#include "rtr/pdu.c"
 #include "rtr/pdu_handler.c"
-#include "rtr/primitive_reader.c"
-#include "rtr/primitive_writer.c"
 #include "rtr/err_pdu.c"
 #include "rtr/db/delta.c"
 #include "rtr/db/deltas_array.c"
 #include "rtr/db/db_table.c"
-#include "rtr/db/rtr_db_impersonator.c"
+#include "rtr/db/rtr_db_mock.c"
 #include "rtr/db/vrps.c"
-#include "slurm/db_slurm.c"
-#include "slurm/slurm_loader.c"
-#include "slurm/slurm_parser.c"
 #include "thread/thread_pool.c"
 
-/* Helper functions */
+/* Mocks */
+
+MOCK_INT(slurm_apply, 0, struct db_table *base, struct db_slurm **slurm)
+MOCK_ABORT_VOID(db_slurm_destroy, struct db_slurm *db)
+
+MOCK_VOID(output_print_data, struct db_table const *db)
+
+/* Mocks end */
 
 struct expected_pdu {
 	uint8_t pdu_type;
@@ -74,7 +70,7 @@ has_expected_pdus(void)
 
 /*
  * This initializes the database using the test values from
- * db/rtr_db_impersonator.c.
+ * db/rtr_db_mock.c.
  */
 static void
 init_db_full(void)
@@ -90,50 +86,38 @@ init_db_full(void)
 }
 
 static void
-init_reset_query(struct rtr_request *request, struct reset_query_pdu *query)
+init_reset_query(struct rtr_request *request)
 {
-	request->pdu = query;
-	request->bytes_len = 0;
-	query->header.protocol_version = RTR_V1;
-	query->header.pdu_type = PDU_TYPE_RESET_QUERY;
-	query->header.m.reserved = 0;
-	query->header.length = 8;
+	static unsigned char raw[] = { 1, 2, 0, 0, 0, 0, 0, 8 };
+
+	request->fd = 0;
+	strcpy(request->client_addr, "192.0.2.1");
+	request->pdu.rtr_version = RTR_V1;
+	request->pdu.type = PDU_TYPE_RESET_QUERY;
+	request->pdu.raw.bytes = raw;
+	request->pdu.raw.bytes_len = sizeof(raw);
+	request->eos = true;
 }
 
 static void
-init_serial_query(struct rtr_request *request, struct serial_query_pdu *query,
-    uint32_t serial)
+init_serial_query(struct rtr_request *request, uint32_t serial)
 {
-	request->pdu = query;
-	request->bytes_len = 0;
-	query->header.protocol_version = RTR_V1;
-	query->header.pdu_type = PDU_TYPE_SERIAL_QUERY;
-	query->header.m.session_id = get_current_session_id(RTR_V1);
-	query->header.length = 12;
-	query->serial_number = serial;
+	static unsigned char raw[] = { 1, 1, 0, 0, 0, 0, 0, 12, 0, 0, 0, 0 };
+
+	request->fd = 0;
+	strcpy(request->client_addr, "192.0.2.1");
+	request->pdu.rtr_version = RTR_V1;
+	request->pdu.type = PDU_TYPE_SERIAL_QUERY;
+	request->pdu.obj.sq.session_id = get_current_session_id(RTR_V1);
+	request->pdu.obj.sq.serial_number = serial;
+	request->pdu.raw.bytes = raw;
+	request->pdu.raw.bytes_len = sizeof(raw);
+	request->eos = true;
 }
 
-/* Impersonator functions */
+/* Mocks */
 
-unsigned int
-config_get_deltas_lifetime(void)
-{
-	return 5;
-}
-
-int
-clients_set_rtr_version(int fd, uint8_t rtr_version)
-{
-	return 0;
-}
-
-int
-clients_get_rtr_version_set(int fd, bool *is_set, uint8_t *rtr_version)
-{
-	(*is_set) = true;
-	(*rtr_version) = RTR_V1;
-	return 0;
-}
+MOCK_UINT(config_get_deltas_lifetime, 5, void)
 
 int
 send_cache_reset_pdu(int fd, uint8_t version)
@@ -219,7 +203,7 @@ send_end_of_data_pdu(int fd, uint8_t version, serial_t end_serial)
 
 int
 send_error_report_pdu(int fd, uint8_t version, uint16_t code,
-    struct rtr_request const *request, char *message)
+    struct rtr_buffer const *request, char *message)
 {
 	pr_op_info("    Server sent Error Report %u: '%s'", code, message);
 	ck_assert_int_eq(pop_expected_pdu(), PDU_TYPE_ERROR_REPORT);
@@ -232,15 +216,12 @@ send_error_report_pdu(int fd, uint8_t version, uint16_t code,
 START_TEST(test_start_or_restart)
 {
 	struct rtr_request request;
-	struct reset_query_pdu client_pdu;
 
 	pr_op_info("-- Start or Restart --");
 
-	/* Prepare DB */
+	/* Init */
 	init_db_full();
-
-	/* Init client request */
-	init_reset_query(&request, &client_pdu);
+	init_reset_query(&request);
 
 	/* Define expected server response */
 	expected_pdu_add(PDU_TYPE_CACHE_RESPONSE);
@@ -250,7 +231,7 @@ START_TEST(test_start_or_restart)
 	expected_pdu_add(PDU_TYPE_END_OF_DATA);
 
 	/* Run and validate */
-	ck_assert_int_eq(0, handle_reset_query_pdu(0, &request));
+	ck_assert_int_eq(0, handle_reset_query_pdu(&request));
 	ck_assert_uint_eq(false, has_expected_pdus());
 
 	/* Clean up */
@@ -262,26 +243,23 @@ END_TEST
 START_TEST(test_typical_exchange)
 {
 	struct rtr_request request;
-	struct serial_query_pdu client_pdu;
 
 	pr_op_info("-- Typical Exchange --");
 
-	/* Prepare DB */
+	/* Init */
 	init_db_full();
-
-	/* From serial 0: Init client request */
-	init_serial_query(&request, &client_pdu, 0);
+	init_serial_query(&request, 0);
 
 	/* From serial 0: Define expected server response */
 	/* Server doesn't have serial 0. */
 	expected_pdu_add(PDU_TYPE_CACHE_RESET);
 
 	/* From serial 0: Run and validate */
-	ck_assert_int_eq(0, handle_serial_query_pdu(0, &request));
+	ck_assert_int_eq(0, handle_serial_query_pdu(&request));
 	ck_assert_uint_eq(false, has_expected_pdus());
 
 	/* From serial 1: Init client request */
-	init_serial_query(&request, &client_pdu, 1);
+	init_serial_query(&request, 1);
 
 	/* From serial 1: Define expected server response */
 	expected_pdu_add(PDU_TYPE_CACHE_RESPONSE);
@@ -294,11 +272,11 @@ START_TEST(test_typical_exchange)
 	expected_pdu_add(PDU_TYPE_END_OF_DATA);
 
 	/* From serial 1: Run and validate */
-	ck_assert_int_eq(0, handle_serial_query_pdu(0, &request));
+	ck_assert_int_eq(0, handle_serial_query_pdu(&request));
 	ck_assert_uint_eq(false, has_expected_pdus());
 
 	/* From serial 2: Init client request */
-	init_serial_query(&request, &client_pdu, 2);
+	init_serial_query(&request, 2);
 
 	/* From serial 2: Define expected server response */
 	expected_pdu_add(PDU_TYPE_CACHE_RESPONSE);
@@ -308,18 +286,18 @@ START_TEST(test_typical_exchange)
 	expected_pdu_add(PDU_TYPE_END_OF_DATA);
 
 	/* From serial 2: Run and validate */
-	ck_assert_int_eq(0, handle_serial_query_pdu(0, &request));
+	ck_assert_int_eq(0, handle_serial_query_pdu(&request));
 	ck_assert_uint_eq(false, has_expected_pdus());
 
 	/* From serial 3: Init client request */
-	init_serial_query(&request, &client_pdu, 3);
+	init_serial_query(&request, 3);
 
 	/* From serial 3: Define expected server response */
 	expected_pdu_add(PDU_TYPE_CACHE_RESPONSE);
 	expected_pdu_add(PDU_TYPE_END_OF_DATA);
 
 	/* From serial 3: Run and validate */
-	ck_assert_int_eq(0, handle_serial_query_pdu(0, &request));
+	ck_assert_int_eq(0, handle_serial_query_pdu(&request));
 	ck_assert_uint_eq(false, has_expected_pdus());
 
 	/* Clean up */
@@ -331,21 +309,18 @@ END_TEST
 START_TEST(test_no_incremental_update_available)
 {
 	struct rtr_request request;
-	struct serial_query_pdu serial_query;
 
 	pr_op_info("-- No Incremental Update Available --");
 
-	/* Prepare DB */
+	/* Init */
 	init_db_full();
-
-	/* Init client request */
-	init_serial_query(&request, &serial_query, 10000);
+	init_serial_query(&request, 10000);
 
 	/* Define expected server response */
 	expected_pdu_add(PDU_TYPE_CACHE_RESET);
 
 	/* Run and validate */
-	ck_assert_int_eq(0, handle_serial_query_pdu(0, &request));
+	ck_assert_int_eq(0, handle_serial_query_pdu(&request));
 	ck_assert_uint_eq(false, has_expected_pdus());
 
 	/* The Reset Query is already tested in start_or_restart. */
@@ -359,32 +334,30 @@ END_TEST
 START_TEST(test_cache_has_no_data_available)
 {
 	struct rtr_request request;
-	struct serial_query_pdu serial_query;
-	struct reset_query_pdu reset_query;
 
 	pr_op_info("-- Cache Has No Data Available --");
 
-	/* Prepare DB */
+	/* Init */
 	ck_assert_int_eq(0, vrps_init());
 
 	/* Serial Query: Init client request */
-	init_serial_query(&request, &serial_query, 0);
+	init_serial_query(&request, 0);
 
 	/* Serial Query: Define expected server response */
 	expected_pdu_add(PDU_TYPE_ERROR_REPORT);
 
 	/* Serial Query: Run and validate */
-	ck_assert_int_eq(0, handle_serial_query_pdu(0, &request));
+	ck_assert_int_eq(0, handle_serial_query_pdu(&request));
 	ck_assert_uint_eq(false, has_expected_pdus());
 
 	/* Reset Query: Init client request */
-	init_reset_query(&request, &reset_query);
+	init_reset_query(&request);
 
 	/* Reset Query: Define expected server response */
 	expected_pdu_add(PDU_TYPE_ERROR_REPORT);
 
 	/* Reset Query: Run and validate */
-	ck_assert_int_eq(0, handle_reset_query_pdu(0, &request));
+	ck_assert_int_eq(0, handle_reset_query_pdu(&request));
 	ck_assert_uint_eq(false, has_expected_pdus());
 
 	/* Clean up */
@@ -395,22 +368,19 @@ END_TEST
 START_TEST(test_bad_session_id)
 {
 	struct rtr_request request;
-	struct serial_query_pdu client_pdu;
 
 	pr_op_info("-- Bad Session ID --");
 
-	/* Prepare DB */
+	/* Init */
 	init_db_full();
-
-	/* From serial 0: Init client request */
-	init_serial_query(&request, &client_pdu, 0);
-	client_pdu.header.m.session_id++;
+	init_serial_query(&request, 0);
+	request.pdu.obj.sq.session_id++;
 
 	/* From serial 0: Define expected server response */
 	expected_pdu_add(PDU_TYPE_ERROR_REPORT);
 
 	/* From serial 0: Run and validate */
-	ck_assert_int_eq(-EINVAL, handle_serial_query_pdu(0, &request));
+	ck_assert_int_eq(-EINVAL, handle_serial_query_pdu(&request));
 	ck_assert_uint_eq(false, has_expected_pdus());
 
 	/* Clean up */
@@ -418,22 +388,7 @@ START_TEST(test_bad_session_id)
 }
 END_TEST
 
-size_t
-serialize_serial_query_pdu(struct serial_query_pdu *pdu, unsigned char *buf)
-{
-	unsigned char *ptr;
-
-	ptr = buf;
-	ptr = write_int8(ptr, pdu->header.protocol_version);
-	ptr = write_int8(ptr, pdu->header.pdu_type);
-	ptr = write_int16(ptr, pdu->header.m.session_id);
-	ptr = write_int32(ptr, pdu->header.length);
-	ptr = write_int32(ptr, pdu->serial_number);
-
-	return ptr - buf;
-}
-
-Suite *pdu_suite(void)
+static Suite *pdu_suite(void)
 {
 	Suite *suite;
 	TCase *core, *error;
@@ -459,8 +414,6 @@ int main(void)
 	SRunner *runner;
 	int tests_failed;
 
-	log_setup(true);
-
 	suite = pdu_suite();
 
 	runner = srunner_create(suite);
